#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"
#include "mtf.h"
#include "fft.h"

#include <time.h>
long loadingtime;

int method;

int usage(char *prgname)
{
    fprintf(stderr, "%s - Apply a bandpass filter to an arbitary image matrix\n", prgname);
    fprintf(stderr, "Usage:\n\t%s ", prgname);
    fprintf(stderr, "[-low <0..1:double,default=0.333>] [-high <0..1:double> default=0.5] [-ideal|-exponential (default=ideal)] [-mink <double> default=2.0] [input_matrix_file_name] \n");
    exit (EXIT_FAILURE);
}

int main(int argc, char *argv[])
{
    matinfo In, Mask, (*mtf_func)();
    matrix Re, Im;
    char *filename;
    int argn=1;

    double  low = 0.1, high = 0.2;
    double band_center = 0.333;
    double mink = 2.0;
    mtf_func = gen_idealMTF;

    /*
    int col_stand = False;
    */

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        else if (keymatch(argv[argn], "-low")) {
            if (sscanf(argv[++argn], "%lg", &low) != 1)  usage(argv[0]);
	    if ( (low < 0.0) || (low > 1.0) ) {
		fprintf(stderr, "Illegal value low=%g",low);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-high")) {
            if (sscanf(argv[++argn], "%lg", &high) != 1)  usage(argv[0]);
	    if ( (high < 0.0) || (high > 1.0) ) {
		fprintf(stderr, "Illegal value high=%g", high);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-exponential")) {
	    mtf_func = gen_exponentialMTF;
        } else if (keymatch(argv[argn], "-ideal")) {
	    mtf_func = gen_idealMTF;
        } else if (keymatch(argv[argn], "-mink")) {
            if (sscanf(argv[++argn], "%lg", &mink) != 1)  usage(argv[0]);
	    if ( mink < 0.0 ) {
		fprintf(stderr, "Illegal value mink=%g", mink);
		exit (EXIT_FAILURE);
	    }
	} else if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }

    if (argn != argc) { 
        if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory at main() in %s\n", argv[0]); 
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy() at main() in %s\n", argv[0]);
            exit (EXIT_FAILURE);
        }
        filename[strlen(argv[argn])] = '\0';
        argn++;
    } else {
        filename = (char *)malloc(2);
        filename[0]='-'; filename[1]='\0';
    }
    if (argn != argc) usage(argv[0]);

    In = matinfo_read(filename);
    Re = matCpy(In->mat,In->n,In->m);
    Im = matinit(In->n, In->m);
    fft2d(In->n, Re, Im, 1);
    Mask = mtf_func(In->n, In->m, 0.333, low, high, 1, mink);
    matMASK(Re, In->n, In->m, Mask->mat, In->n, In->m);
    matMASK(Im, In->n, In->m, Mask->mat, In->n, In->m);
    fft2d(In->n, Re, Im, -1);

#ifdef DEBUG2
    fwritemat(stdout,Re, In->n, In->m, "","");
    exit (EXIT_FAILURE);
#endif

    fprintf(stdout, "### ProgramName: %s \n", argv[0]);
    fprint_signature(stdout);
    fwritemat(stdout, Re, In->n, In->m, "### Result\n", "");

    if ( low > high ) {
	fprintf(stderr, "value of low is greater than that of high\n");
	exit (EXIT_FAILURE);
    }
    band_center = (high + low)/2.0;
    return OK;
}
