#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"
#include "mtf.h"

double modulate_idealMTF(double distance, double band_center, double low, double high) 
{
    if ( (distance >= low) && (distance <= high) ) return 1.0;
    else                                           return 0.0;
}


matinfo gen_idealMTF(int n, int m,  double band_center, double low, double high, int col_stand, double mink)
{
    int i,j;
    int rows_center, cols_center;
    matrix distr, distc;
		scalar scale, distance;
    matinfo Ret;

    rows_center = n >> 1;
    cols_center = m >> 1;
    scale = (col_stand) ?  cols_center : rows_center;

    distr = gen_distance_table(rows_center, scale, mink);
    distc = gen_distance_table(cols_center, scale, mink);

    if ( (Ret = (matinfo)malloc(sizeof(matinfo))) == NULL) {
        fprintf(stderr, "Could not allocate memory at gen_MTF()\n");
        exit (EXIT_FAILURE);
    }
    if ( (Ret->mat = matinit(n, m)) == NULL) {
        fprintf(stderr, "Could not allocate memory at gen_MTF()\n");
        exit (EXIT_FAILURE);
    }
    Ret->n = n;
    Ret->m = m;

    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            distance = calc_distance(i, j, rows_center, cols_center, distr, distc, mink);
            *(Ret->mat+offs(Ret->n,Ret->m,i,j)) = modulate_idealMTF(distance, band_center, low, high);
	    /*
            *(Ret->mat+offs(Ret->n,Ret->m,i,j)) = modulate_exponentialMTF(distance, band_center, low, high);
	    */

        }
    }
    free (distr); free (distc);
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *prgname)
{
    fprintf(stderr, "%s - Generate Modulation Transfer Function\n", prgname);
    fprintf(stderr, "Usage: %s [-nrows <int> default=128] [-ncols <int> default=128] [-low <0..1:double> default=0.333] [-high <0..1:double> default=0.5] [-mink <double> default=2.0]\n", prgname);
    exit (EXIT_FAILURE);
}

int main (int argc, char *argv[])
{
    matinfo In;
    int argn=1;
    int col_stand = False;
    int ncols = 128, nrows = 128;
    double  low = 0.167, high = 0.500;
    double band_center = 0.333; /* or 0.5, 0.67 */
    double mink = 2.0;

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        else if (keymatch(argv[argn], "-low")) {
            if (sscanf(argv[++argn], "%lg", &low) != 1)  usage(argv[0]);
	    if ( (low < 0.0) || (low > 1.0) ) {
		fprintf(stderr, "Illegal value low=%g",low);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-high")) {
            if (sscanf(argv[++argn], "%lg", &high) != 1)  usage(argv[0]);
	    if ( (high < 0.0) || (high > 1.0) ) {
		fprintf(stderr, "Illegal value high=%g", high);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-nrows")) {
            if (sscanf(argv[++argn], "%d", &nrows) != 1)  usage(argv[0]);
	    if ( nrows <= 0 ) {
		fprintf(stderr, "Illegal value nrows=%d", nrows);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-ncols")) {
            if (sscanf(argv[++argn], "%d", &ncols) != 1)  usage(argv[0]);
	    if ( ncols <= 0 ) {
		fprintf(stderr, "Illegal value nrows=%d", ncols);
		exit (EXIT_FAILURE);
	    }
        } else if (keymatch(argv[argn], "-mink")) {
            if (sscanf(argv[++argn], "%lg", &mink) != 1)  usage(argv[0]);
	    if ( mink < 0.0 ) {
		fprintf(stderr, "Illegal value mink=%g", mink);
		exit (EXIT_FAILURE);
	    }
	}
        ++argn;
    }

    if ( low > high ) {
	fprintf(stderr, "value of low is greater than that of high\n");
	exit (EXIT_FAILURE);
    }
    band_center = (high + low)/2.0;

    In = gen_idealMTF(nrows, ncols, band_center, low, high, col_stand, mink);
    fprintf(stdout, "### ProgramName: %s -nrows %d -ncols %d -low %g -high %g\n", argv[0], nrows, ncols, low, high);
    fprint_signature(stdout);

    fwritemat (stdout, In->mat, In->n, In->m, "### Result\n", "");
    fwritemat (stdout, In->mat, In->n, In->m, "", "");
    return OK;
}
#endif STANDALONE
