#include <stdlib.h>
#include "general.h"
#include "fileio_util.h"

/* Both STR_LNG and ABS_STR_LNG are described in fileio_util.h */

/* Get a line from input stream, 
   if the input stream is illegal, then return NULL */
char *getline(FILE *fp)
{
    static char *stre = NULL;
    static long strl = 0;

    long len;
    int c;
    char *tstr;
    
    if (stre == NULL) {
        strl = STR_LNG;
        stre = (char *) malloc(sizeof(char) * strl);
        if (stre == NULL) {
            fileio_util_perror("Could not allocate memory at getline()");
            return NULL;
        }
    }
    len = 0;
    tstr = stre;
    
    while (1) {
        /* If line buffer is too small, increase its size */
        if (len >= strl) {
            strl += STR_LNG;
            tstr = stre = realloc(stre, sizeof(char) * strl);
            if (stre == NULL) {
                fileio_util_perror( "Could not reallocate memory at getline()\n");
                return NULL;
            }
            if (strl > ABS_STR_LNG) {
                fprintf(stderr, "Too long lines in file (max %d) at getline()\n", ABS_STR_LNG);
                return NULL;
            }
        }

        /* get next character */
        c = fgetc(fp);

        /* end of line? */
        if (c == '\n') {
            tstr[len] = '\0';
            break;
        }

        /* EOF? */
        if (c == EOF) {
            tstr[len] = '\0';
            /* really EOF? */
            if (feof(fp)) {
                if (len == 0)
                    tstr = NULL;
            } else {
                tstr = NULL;
            }
            break;
        }
        tstr[len++] = c;
    }
    return tstr;
}
