#include <stdlib.h>
#include <math.h>
#include <curses.h>
#include <term.h>
#include <time.h>

#define YES (1==1)
#define NO  (!YES)

int ON  =  1.0;
int OFF = -1.0;

int FileF = NO;
char filename[64];

int  ROW = 32, COL = 32, UNIT;
int  InitPatN=0, NPattern=0;
int  InitDev = 10;   /* default initial hammin distance from initial pattern */
int  Interval = 100, MaxIter = 1000;
int  *Count;

double  *X;         /* Output of each unit */
double  *PAT;       /* Data Pattern to be memorized */
double  *Wgt;       /* Weight Matrix among Units */
double  *EnergyN;

long loadingtime;
long SEED;

int Usage(unsigned char *prg) 
{
    fprintf(stderr,"Usage:\n");
    fprintf(stderr,"%s -f Input_Pattern_File -n Number_of_Patterns -x columns -y rows [-p Initial_Pattern] [-d Initial_Deviation] [-t Max_iteration] [-r seed_of_random_number] [-i observation_interval]\n", prg);
    exit (EXIT_FAILURE);
}


void initialize_term(void)
{
    char *term;
    int status;
    
    if ((term = getenv("TERM")) == NULL){
        fprintf(stderr,"TERM in not set");
        exit(EXIT_FAILURE);
    }
    setupterm(term, 1, &status);
    if (status != 1) {
        fprintf(stderr,"Error in setupterm");
        exit(EXIT_FAILURE);
    }
    if (clear_screen == NULL){
        fprintf(stderr,"clear string is not set\n");
        exit(EXIT_FAILURE);
    }
}

int irand(int from, int to)
{
    return (from + (rand() % (to-from+1)));
}

int Hamming(double a[UNIT], double b[UNIT])
{
    int i;
    double dist;

    dist = 0.0;
    for (i=0; i < UNIT; i++){
	if ( a[i] * b[i] < 1.0 ) {
	    dist += 1.0;
	}
    }
    return (int)dist;
}

int ParseArgs(int argc, char **argv)
{
    int t;
    
    
    for (t=1; t<argc; t++){
        if (argv[t][0] == '-'){
            switch(argv[t][1]){
            case 'f':
                FileF = YES;
                strcpy(filename, argv[++t]);
                break;
            case 'n':
                NPattern = atoi(argv[++t]);
                break;
            case 'r':
		SEED = atoi(argv[++t]);
                srand((unsigned)SEED);
                break;
            case 'p':
                InitPatN = atoi(argv[++t]);
                break;
            case 'd':
                InitDev = atoi(argv[++t]);
                break;
            case 'i':
                Interval = atoi(argv[++t]);
                break;
            case 't':
                MaxIter = atoi(argv[++t]);
                break;
            case 'x':
                COL = atoi(argv[++t]);
                fprintf(stderr, "colums =%d\n", COL);
                break;
            case 'y':
                ROW = atoi(argv[++t]);
                fprintf(stderr, "rows =%d\n", ROW);
                break;
            case 'h':
            default :
                Usage(argv[0]);
                break;
            }
        }
    }
    UNIT = ROW * COL;

    if ( NPattern < 1 ) {
	fprintf(stderr, "Invalid Patterns=%d\n", NPattern);
	Usage(argv[0]);
    }
    if (InitPatN < 0 || InitPatN > NPattern) {
	fprintf(stderr, "Invalid Initial pattern=%d\n",InitPatN);
	Usage(argv[0]);
    }
    if (InitDev < 0 || InitDev >= UNIT) {
	fprintf(stderr, "Invalid distance=%d\n", InitDev);
	Usage(argv[0]);
    }
    if (Interval > MaxIter) {
	fprintf(stderr, "Invalid interval=%d\n", Interval);
	Usage(argv[0]);
    }

    return OK;
}

void ReadPatternFile (char *filename)
{
    FILE *fp;
    int i,j;
    
    if ((fp = fopen(filename, "r")) == NULL) {
        fprintf(stderr, "Invalid Pattern File %s\n", filename);
        exit(EXIT_FAILURE);
    }
    for (i=0; i< NPattern; i++) {
        for (j=0; j<UNIT; j++) {
        skip:
            switch ( fgetc(fp) ){
            case '1':
                PAT[i*UNIT+j] = ON;
                break;
            case '0':
                PAT[i*UNIT+j] = OFF;
                break;
            case EOF:
                fprintf(stderr, "Pattern file inadequate in i=%d,j=%d\n",i,j);
                exit(EXIT_FAILURE);
            default:
                goto skip;
            }
        }
    }
    fclose(fp);
}

void SetInitialPattern (int InitPatN, int InitDev)
{
    int i,j, dist;

    for (i=0; i<UNIT; i++) {
        X[i] = PAT[InitPatN*UNIT+i];
        Count[i] = 0;
    }
    dist = 0;
    for (i=0; dist<InitDev; i++){
        j = irand(0, UNIT-1);
	if ( PAT[InitPatN * UNIT + j] == X[j] ) {
	    X[j] *= OFF;
	} 
	dist = Hamming(X, &PAT[InitPatN*UNIT]);
    }
}

void Initialize(void)
{
    PAT = (double *)calloc(NPattern*UNIT, sizeof(double));
    X = (double *)calloc(UNIT, sizeof(double));
    Wgt = (double *)calloc(UNIT*UNIT, sizeof(double));
    Count = (int *)calloc(UNIT, sizeof(int));
    EnergyN = (double *)calloc(NPattern, sizeof(double));
}


double CalcEnergy(double v[UNIT])
{
    int i,j;
    double energy;
    
    energy = 0.0;
    for (i=0; i < UNIT; i++) {
        for (j=i+1; j< UNIT; j++) {
	    /*
            energy -= Wgt[i*UNIT+j] * v[i] * v[j];
	    */
            energy += - Wgt[i*UNIT+j] * v[i] * v[j];
        }
	/*
        energy -= Wgt[i*UNIT+i] * v[i];
	*/
        energy += - Wgt[i*UNIT+i] * v[i];
    }
    return (energy);
}

void DisplayUnits(void)
{
    int i,j;
    
    for (i=0; i<ROW; i++) {
        for (j=0; j<COL; j++) {
            if (X[i*COL+j] == ON)
                putchar('*');
            else
                putchar(' ');
        }
        putchar('\n');
    }
}

void DisplayCounter(void)
{
    int i,j;
    
    for (i=0; i<ROW; i++) {
        for (j=0; j<COL; j++) {
            printf("%4d", Count[i*COL+j]);
        }
        putchar('\n');
    }
}

void DisplayDistance(void)
{
    int i;
    
    printf("Pat.: Dist Energy\n");
    for (i=0; i<NPattern; i++){
        printf("[%2d] : %3d %g\n", i, Hamming(X, &PAT[i*UNIT]), EnergyN[i]);
    }
}

void Update (void)
{
    int i, iter;
    int target;
    double u;

    for (iter=0; iter< MaxIter; iter++){
        if (iter % Interval == 0) {
            putp(clear_screen);
            printf("iter=%5d\tEnergy=%g\n", iter, CalcEnergy(X));
            DisplayUnits();
            /*
            DisplayCounter();
            DisplayDistance();
            */
        }
        target = irand(0, UNIT-1);
        Count[target]++;
        u = 0.0;
        for (i=0; i < UNIT; i++){
            u += Wgt[target*UNIT+i] * X[i];
        }
        if (u > 0.0) X[target] = ON;
        else         X[target] = OFF;
    }
}

void CalcWeight(void)
{
    int i,j,k;
    
    for (i=0; i<UNIT; i++) {
        Wgt[i*UNIT+i] = 0.0;
        for (j=i+1; j<UNIT; j++){
            Wgt[i*UNIT+j] = 0.0;
            for (k=0; k< NPattern; k++){
                Wgt[i*UNIT+j] += PAT[k*UNIT+i] * PAT[k*UNIT+j];
		/*
                Wgt[i*UNIT+j] += (2.0 * PAT[k*UNIT+i]-1.0) 
		               * (2.0 * PAT[k*UNIT+j]-1.0);
                */
            }
	    Wgt[j*UNIT+i] /= (double)NPattern;
            Wgt[j*UNIT+i] = Wgt[i*UNIT+j];
        }
    }
    for (i=0; i<NPattern; i++){
        EnergyN[i] = CalcEnergy(&PAT[i*UNIT]);
    }
}


int main(int argc, char **argv)
{
    SEED = time(&loadingtime);
    srand((unsigned)SEED);

    initialize_term();
    
    ParseArgs(argc, argv);
    Initialize();

    if ( FileF == YES )
        ReadPatternFile(filename);
    else 
        Usage(argv[0]);

    SetInitialPattern(InitPatN, InitDev);
    CalcWeight();
    
    Update();
    fprintf(stderr, "SEED=%ld\n", SEED);
    exit(EXIT_SUCCESS);
}
