/* 
 * gengrat: generate grating pattern
 * 1993/03/28 by Shinichi Asakawa
 */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "matrix_util.h"
#include "fileio_util.h"

void usage(char *prgname)
{
    fprintf(stderr, "%s: generate a grating pattern\n", prgname);
    fprintf(stderr, "Usage: \n");
    fprintf(stderr, "%s total_size freq [phase] [[amp] [[offset]]]\n", prgname);
    fprintf(stderr, "grating = offset + amp * sin(2 Pi (x + phase)/freq)\n");
    exit (EXIT_FAILURE);
}

double grating_func(double x, double Freq, double Phase, double Amp, double Offset) 
{
    return Offset + Amp * sin(2.0 * Pi * (x + Phase) / Freq );
}

matrix gen_grat(int Size, double Freq, double Phase, double Amp, double Offset) 
{
    int i,j;
    matrix Ret;
    
    Ret = matinit(Size,Size);
    for (i=1; i<=Size; i++) {
	for (j=1; j<=Size; j++) {
	    *(Ret+offs(Size,Size,i,j)) = grating_func(j,Freq,Phase,Amp,Offset);
	}
    }
    return Ret;
}

int main(int argc, char **argv)
{
    int argn;
    int Size=0;
    scalar Freq=1.0, Phase=0.0, Amp=1.0, Offset=0.0;
    matrix Ret;

    if (argc == 1) usage(argv[0]);
    /*
    argn=1;
    while (argv[argn][0] == '-' && argv[argn][1] != '\0') {
    */
    for (argn=1;argv[argn][0] == '-' && argv[argn][1] != '\0'; argn++) {
        if (keymatch(argv[argn], "--help")) { usage(argv[0]); }
        if (keymatch(argv[argn], "-h")) { usage(argv[0]); }
	/*        argn++; */
    }

    switch (argc - argn) {
    case 2:
	Size = atoi(argv[argn++]);
	Freq = atof(argv[argn++]);
	break;
    case 3:
	Size = atoi(argv[argn++]);
	Freq = atof(argv[argn++]);
	Phase = atof(argv[argn++]);
	break;
    case 4:
	Size = atoi(argv[argn++]);
	Freq = atof(argv[argn++]);
	Phase = atof(argv[argn++]);
	Amp = atof(argv[argn++]);
	break;
    case 5:
	Size = atoi(argv[argn++]);
	Freq = atof(argv[argn++]);
	Phase = atof(argv[argn++]);
	Amp = atof(argv[argn++]);
	Offset = atof(argv[argn++]);
	break;
    default:
	usage(argv[0]);
    }

    Ret = gen_grat(Size,Freq,Phase,Amp,Offset);

	fprintf(stdout, "### ProgramName: %s %d %g %g %g %g\n",
					argv[0], Size, Freq, Phase, Amp, Offset);
	fprint_signature(stdout);

    fwritemat(stdout,Ret,Size,Size,"","");
    return OK;
}
