#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

int zerocross(matrix O, int n, int m, matrix D)
{
    int i,j;
    double w1,w2,w3,w4;

    if ( O == NULL ) {
        fprintf(stderr, "Invalid data matrix in zerocross()\n");
        exit (EXIT_FAILURE);
    }
    if ( D == NULL ) {
        fprintf(stderr, "Invalid output matrix in zerocross()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            w1 = 0.0; w2 = 0.0; w3 = 0.0; w4 = 0.0;
            if ( i >= 2 ) {
                w1 = *(O+offs(n,m,i,j)) * *(O+offs(n,m,i-1,j));
            }
            if ( i <= n-1 ) {
                w2 = *(O+offs(n,m,i,j)) * *(O+offs(n,m,i+1,j));
            }
            if ( j >= 2 ) {
                w3 = *(O+offs(n,m,i,j)) * *(O+offs(n,m,i,j-1));
            }
            if ( j <= n-1 ) {
                w4 = *(O+offs(n,m,i,j)) * *(O+offs(n,m,i,j+1));
            }
            if ( (w1 < 0) || (w2 < 0) || (w3 < 0) || (w4 < 0) ) {
                *(D+offs(n,m,i,j)) = 1.0;
            } else {
                *(D+offs(n,m,i,j)) = 0.0;
            }
        }
    }
    return OK;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s detect zero crossing\n", progname);
    fprintf(stderr,"Usage: %s [datafile]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    char *prgname = argv[0], *filename;
    int argn=1;
    matinfo Din;
    matrix Out;
    
    if ((argc != 1) && (argc != 2)) { usage(prgname); } 
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(prgname);
        else if (keymatch(argv[argn], "-h")) usage(prgname);
        ++argn;
    }
    
    if (argc == 2) {
        if ((filename = (char *)malloc(strlen(argv[1])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory at main() in zerocross\n");
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[1]) == NULL) {
            fprintf(stderr, "Could not strcpy at main() in zerocross\n");
            exit (EXIT_FAILURE);
        }
				filename[strlen(argv[1])] = '\0';
    } else {
        if ((filename = (char *)malloc(2)) == NULL) {
            fprintf(stderr, "Could not allocate memory at zerocross\n");
            exit (EXIT_FAILURE);
        }
        filename[0]='-'; filename[1]='\0';
    }


    Din = matinfo_read(filename);
    if ((Din->n == 0) || (Din->m == 0) || (Din->mat == NULL) ) {
        fprintf(stderr, "Could not read data matrix in zerocross main()\n");
        usage(prgname);
    }
    
    Out = matinit(Din->n,Din->m);
    zerocross(Din->mat, Din->n, Din->m, Out);
    
    fprintf(stdout, "### ProgramName: %s %s\n", prgname, filename);
    fprint_signature(stdout);
    fprintf(stdout, "### filename=%s,(%d,%d)\n", filename, Din->n, Din->m);
    
    fwritemat(stdout, Out, Din->n, Din->m, "", "%g ");
    return EXIT_SUCCESS;
}
#endif STANDALONE
