#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <float.h>   /* for DBL_MAX */
#include "matrix_util.h"

matrix bstat_min (matrix A, int n, int m)
{
    int i,j;
    matrix ret;
    
    ret = matinit(1,m);
    matfill(ret, 1, m, DBL_MAX);
    
    for (j=1; j<=m; j++)
        for (i=1; i<=n; i++)
            if ( *(A+offs(n,m,i,j)) < *(ret+offs(1,m,1,j)) )
                *(ret+offs(1,m,1,j)) = *(A+offs(n,m,i,j));
    return ret;
}

matrix bstat_max (matrix A, int n, int m)
{
    int i,j;
    matrix ret;
    
    ret = matinit(1,m);
    matfill(ret,1, m, -DBL_MAX + 1.0);
    
    for (i=1; i<=n; i++)
        for (j=1; j<=m; j++)
            if ( *(A+offs(n,m,i,j)) > *(ret+offs(1,m,1,j)) )
                *(ret+offs(1,m,1,j)) = *(A+offs(n,m,i,j));
    return ret;
}

matrix bstat_range (matrix A, int n, int m)
{
    int i,j;
    matrix ret = matinit(1,m), mina, maxa;
    
    mina = bstat_min(A,n,m);         
    maxa = bstat_max(A,n,m);         
    for (j=1; j<=m; j++)
        for (i=1; i<=n; i++)
            *(ret+offs(1,m,1,j)) = *(maxa+offs(n,m,1,j)) - *(mina+offs(n,m,1,j));
    free(mina); free(maxa);
    return ret;
}

matrix bstat_square (matrix A, int n, int m)
{
    int i,j;
    matrix ret= matinit(1,m);
    
    for (j=1; j<=m; j++)
        for (i=1; i<=n; i++)
            *(ret+offs(1,m,1,j)) += *(A+offs(n,m,i,j)) * *(A+offs(n,m,i,j));
    return ret;
}

matrix bstat_sum (matrix A, int n, int m)
{
    int i,j;
    matrix sum = matinit(1,m);
    
    for (j=1; j<=m; j++)
        for (i=1; i<=n; i++)
            *(sum+offs(1,m,1,j)) += *(A+offs(n,m,i,j));
    return sum;
}

matrix bstat_mean (matrix A, int n, int m)
{
    int i,j;
    matrix mean = matinit(1,m);
    
    for (j=1; j<=m; j++)
        for (i=1; i<=n; i++)
            *(mean+offs(1,m,1,j)) += *(A+offs(n,m,i,j))/(double)n;
    return mean;
}

matrix bstat_variance (matrix A, int n, int m)
{
    int i,j;
    scalar wj;
    matrix ret = matinit(1,m), avg;
    
    avg = bstat_mean(A,n,m);
    for (j=1; j<=m; j++) {
        for (i=1; i<=n; i++) {
            wj = *(A+offs(n,m,i,j)) - *(avg+offs(1,m,1,j));
            *(ret+offs(1,m,1,j)) += wj * wj / (double)n;
        }
    }
    free(avg);
    return ret;
}

matrix bstat_sd (matrix A, int n, int m)
{
    int i,j;
    scalar wj;
    matrix ret = matinit(1,m), avg;
    
    avg = bstat_mean(A,n,m);
    
    for (j=1; j<=m; j++) {
        for (i=1; i<=n; i++) {
            wj = *(A+offs(n,m,i,j)) - *(avg+offs(1,m,1,j));
            *(ret+offs(1,m,1,j)) += wj * wj / (double)n;
        }
        *(ret+offs(1,m,1,j)) = sqrt(*(ret+offs(1,m,1,j)));
    }
    free(avg);
    return ret;
}

matrix bstat_skew (matrix A, int n, int m, vector mean, vector sd)
{
    int i,j;
    scalar wj;
    matrix ret = matinit(1,m);
    
    for (j=1; j<=m; j++) {
        for (i=1; i<=n; i++) {
            wj = (*(A+offs(n,m,i,j)) - *(mean+offs(1,m,1,j)))/ *(sd+offs(1,m,1,j));
            *(ret+offs(1,m,1,j)) += wj * wj * wj / (double)n;
        }
    }
    return ret;
}

matrix bstat_kurtosis (matrix A, int n, int m, vector mean, vector sd)
{
    int i,j;
    scalar wj;
    matrix ret = matinit(1,m);
    
    for (j=1; j<=m; j++) {
        for (i=1; i<=n; i++) {
            wj = (*(A+offs(n,m,i,j)) - *(mean+offs(1,m,1,j)))/ *(sd+offs(1,m,1,j));
            *(ret+offs(1,m,1,j)) += wj * wj * wj * wj/ (double)n;
        }
    }
    return ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <stdio.h>
#include "fileio_util.h"

FLAG MEAN = False, SD = False, VAR = False, MAX = False, MIN = False;
FLAG SKEW = False, KURT = False;
FLAG RANGE = False, SUM = False, SQ = False;
FLAG CORR = False, COV = False, UBCOV = False;
FLAG CORRINV = False, COVINV = False, UBCOVINV = False;

int usage (void)
{
    fprintf(stderr,"bstat : basic statistics\n");
    fprintf(stderr,"Usage bstat [options] [datafile]\n");
    fprintf(stderr,"Arguments:\n");
    fprintf(stderr,"Options: \n");
    fprintf(stderr," -mean: output means (defalut)\n");
    fprintf(stderr," -sd: output standard deviation\n");
    fprintf(stderr," -max: output maximam values\n");
    fprintf(stderr," -min: output minmum values\n");
    fprintf(stderr," -range: output range\n");
    fprintf(stderr," -skew: output skew\n");
    fprintf(stderr," -kurtosis: output kurotsis\n");
    fprintf(stderr," -corr: output correlation matrix\n");
    fprintf(stderr," -invcorr: output inverse correlation matrix\n");
    fprintf(stderr," -cov: output sample covariance matrix\n");
    fprintf(stderr," -invcov: output inverse sample covariance matrix\n");
    fprintf(stderr," -ubcov: output unbiased covariance matrix\n");
    fprintf(stderr," -invubcov: output inverse unbiased covariance matrix\n");
    fprintf(stderr," -v : verbose output\n");
    fprintf(stderr," -h : show this help\n");
    exit (0);
}

int main(int argc, char **argv)
{
    int m=0, n=0, argn = 1;
    matinfo Ain;
    matrix A, mean, sd, max, min, range, var, sum, sq, skew, kurt;
    matrix Corr, Cov, Ubcov, Covinv, Ubcovinv, Corrinv;
    scalar det;
    FLAG verbose = False;
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "-help")) {
            usage();
        } else if (keymatch(argv[argn], "-v")) {
            verbose = True;
        } else if (keymatch(argv[argn], "-mean")) {
            MEAN = True;
        } else if (keymatch(argv[argn], "-sum")) {
            SUM = True;
        } else if (keymatch(argv[argn], "-square")) {
            SQ = True;
        } else if (keymatch(argv[argn], "-sd")) {
            SD = True;
        } else if (keymatch(argv[argn], "-var")) {
            VAR = True;
        } else if (keymatch(argv[argn], "-max")) {
            MAX = True;
        } else if (keymatch(argv[argn], "-min")) {
            MIN = True;
        } else if (keymatch(argv[argn], "-range")) {
            RANGE = True;
        } else if (keymatch(argv[argn], "-skew")) {
            SKEW = True;
        } else if (keymatch(argv[argn], "-kurotsis")) {
            KURT = True;
        } else if (keymatch(argv[argn], "-corr")) {
            CORR = True;
        } else if (keymatch(argv[argn], "-invcorr")) {
            CORRINV = True;
        } else if (keymatch(argv[argn], "-cov")) {
            COV = True;
        } else if (keymatch(argv[argn], "-ubcov")) {
            UBCOV = True;
        } else if (keymatch(argv[argn], "-invcorr")) {
            CORRINV = True;
        } else if (keymatch(argv[argn], "-invcov")) {
            COVINV = True;
        } else if (keymatch(argv[argn], "-invubcov")) {
            UBCOVINV = True;
        } else {
            usage();
        }
        ++argn;
    }
    
    if (argn != argc) { Ain = matinfo_read(argv[argn]); }
    else              { Ain = matinfo_read("-"); }
    
    A = Ain->mat;
    n = Ain->n;
    m = Ain->m;
    if ((n==0) || (m==0)) usage();
    
    if (verbose)
        MEAN = SD = VAR = MIN = MAX = RANGE = SUM = SQ = SKEW = KURT = True;
    
    if ((MEAN + SD + MAX + MIN + RANGE + VAR + SUM + SQ
         +        CORR + COV + UBCOV + CORRINV + COVINV + UBCOVINV) == 0)
        MEAN = True;
    
    if (MEAN || SKEW || KURT) {
        mean = bstat_mean(A,n,m);
        if ( mean == NULL ) {
            fileio_util_perror("Could not calculate mean");
            exit (EXIT_FAILURE); 
        }
    }
    if (VAR) {
        var = bstat_variance(A,n,m);
        if ( var == NULL ) {
            fileio_util_perror("Could not calculate var");
            exit (EXIT_FAILURE); 
        }
    }
    if (SQ) {
        sq = bstat_square(A,n,m);
        if ( sq == NULL ) {
            fileio_util_perror("Could not calculate sq"); 
	    exit (EXIT_FAILURE); 
        }
    }
    if (SUM) {
        sum = bstat_sum(A,n,m);
        if ( sum == NULL ) {
            fileio_util_perror("Could not calculate sum");
            exit (EXIT_FAILURE); 
        }
    }
    if (SD || SKEW || KURT) {
        sd = bstat_sd(A,n,m);
        if ( sd == NULL ) {
            fileio_util_perror("Could not calculate sd");
            exit (EXIT_FAILURE); 
        }
    }
    if (MAX) {
        max = bstat_max(A,n,m);
        if ( max == NULL ) {
            fileio_util_perror("Could not calculate max"); 
            exit (EXIT_FAILURE); 
        }
    }
    if (MIN) {
        min = bstat_min(A,n,m);
        if ( min == NULL ) {
            fileio_util_perror("Could not calculate min");
            exit (EXIT_FAILURE); 
        }
    }
    if (RANGE) {
        range = bstat_range(A,n,m);
        if ( range == NULL ) {
            fileio_util_perror("Could not calculate range");
            exit (EXIT_FAILURE); 
        }
    }
    if (SKEW) {
        skew = bstat_skew(A,n,m, mean,sd);
        if ( range == NULL ) {
            fileio_util_perror("Could not calculate skey");
            exit (EXIT_FAILURE); 
        }
    }
    if (KURT) {
        kurt = bstat_kurtosis(A,n,m, mean,sd);
        if ( range == NULL ) {
            fileio_util_perror("Could not calculate kurtosis");
            exit (EXIT_FAILURE); 
        }
    }
    
    if (CORR) {
        Corr = matCorr(A,n,m);
        if ( Corr == NULL ) {
            fileio_util_perror("Could not calculate corr");
            exit (EXIT_FAILURE); 
        }
    }
    if (COV) {
        Cov = matCov(A,n,m);
        if ( Cov == NULL ) {
            fileio_util_perror("Could not calculate cov");
            exit (EXIT_FAILURE); 
        }
    }
    if (COVINV) {
        Covinv = matCovinv(A,n,m, &det);
        if ( Covinv == NULL ) {
            fileio_util_perror("Could not calculate co-inv"); 
            exit (EXIT_FAILURE); 
        }
    }
    if (UBCOV) {
        Ubcov = matUbcov(A,n,m);
        if ( Ubcov == NULL ) {
            fileio_util_perror("Could not calculate unbiased covariance");
            exit (EXIT_FAILURE); 
        }
    }
    if (UBCOVINV) {
        Ubcovinv = matUbcovinv(A,n,m, &det);
        if ( Ubcov == NULL ) {
            fileio_util_perror("Could not calculate inversed unbaiased covaniance"); 
            exit (EXIT_FAILURE); 
        }
    }
    if (CORRINV) {
        Corrinv = matCorrinv(A,n,m, &det);
        if ( Corrinv == NULL ) {
            fileio_util_perror("Could not calculate inversed correlation matrix");
            exit (EXIT_FAILURE); 
        }
    }
    fprintf(stdout, "### ProgramName: %s %s (Basic statistics)\n", argv[0], (argn != argc) ? argv[argn] : "-" );
    fprint_signature(stdout);
    if (MEAN) fwritemat(stdout, mean, 1, m, "### mean:\n", "%.6g ");
    if (SUM) fwritemat(stdout, sum,  1, m, "### sum:\n", "%.6g ");
    if (SD)  fwritemat(stdout, sd,   1, m, "### sd:\n",   "%.6g ");
    if (VAR) fwritemat(stdout, var,  1, m, "### variance:\n",  "%.6g ");
    if (SQ)  fwritemat(stdout, sq,   1, m, "### sum of square:\n",  "%.6g ");
    if (MAX) fwritemat(stdout, max,  1, m, "### max:\n",  "%.6g ");
    if (MIN) fwritemat(stdout, min,  1, m, "### min:\n",  "%.6g ");
    if (RANGE) fwritemat(stdout, range,1, m, "### range:\n","%.6g ");
    if (SKEW) fwritemat(stdout, skew,1, m, "### skew:\n","%.6g ");
    if (KURT) fwritemat(stdout, kurt,1, m, "### kurtosis(values < 3 means platykurtic, and values > 3 leptokurtic):\n","%.6g ");
    if (COV) fwritemat(stdout,Cov,m,m,"### covarinace:\n","%.6g ");
    if (CORR) fwritemat(stdout,Corr,m,m,"### correlation matrix:\n","%.6g ");
    if (UBCOV) {
        fwritemat(stdout,Ubcov,m,m,"### unbiased covarinace:\n","%.6g ");
    }
    if (COVINV) {
        fprintf(stdout,"### determinant of data matrix: %.6g\n",det);
        fwritemat(stdout,Covinv,m,m,"### inverse covarinace:\n","%.6g ");
    }
    if (UBCOVINV) {
        fprintf(stdout,"### determinant of data matrix: %.6g\n",det);
        fwritemat(stdout,Ubcovinv,m,m,"### inverse unbiased covarinace:\n","%.6g ");
    }
    if (CORRINV) {
        fprintf(stdout,"### determinant of data matrix: %.6g\n",det);
        fwritemat(stdout,Corrinv,m,m,"### inverse of correlation:\n","%.6g ");
    }
    return EXIT_SUCCESS;
}
#endif STANDALONE
