/*  Calculate eigen values by power method */
#include <stdlib.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"

/* set initial values  */
int inival(vector x, int n)
{
    int i;

    for (i=1 ; i<=n ; i++)
        *(x+offs(n,1,i,1)) =( rand() & 2047 )-4096;
    vectnormal(x,n);
    return OK;
}

/*   $BH?I|%Y%/%H%k$+$i8GM-CM$r;;=P(B   */
int rat(vector x, vector y, double *pmin, double *pmax, double *prq, int n)
{
    int i;
    double q, rmin, rmax;
    double bignum=1.0e20;
  
    rmin = bignum;
    rmax=(-bignum);
  
    for (i=1; i<=n; i++) {
        if (fabs(*(y+offs(n,1,i,1)))<eps) {
            rmin=eps;
        } else {
            if (fabs(*(x+offs(n,1,i,1)))>eps) {
                q = *(y+offs(n,1,i,1)) / *(x+offs(n,1,i,1));
                if (q < rmin) rmin = q;
                if (q > rmax) rmax = q;
            } else {
                rmax = bignum;
            }
        }
    }
    *prq  = vectinnerprd(x,y,n);
    *pmin = rmin;
    *pmax = rmax;
    return OK;
}

/* $BN_>hK!$K$h$k8GM-CM!"8GM-%Y%/%H%k$N7W;;(B */
/* $BF~NO9TNs(B a0 $B$KBP$7$F8GM-CMLdBj$r2r$/(B */
/* $B7W;;7k2L$O0J2<$N$H$*$j(B */
/* $B$^$:(B evv $B$K8GM-CM$,:GBg8GM-CM$+$i=gHV$KBg(B--->$B>.$N=g$K5a$a$i$l$k(B*/
/* $BBP1~$9$k8GM-%Y%/%H%k$,(B vv $B$KJ]B8$5$l$k(B */
/* n $B$OF~NO9TNs$N<!?t(B */
/* p $B$O5a$a$k8GM-CM$N?t(B */  
int eigen_pm(matrix a0, vector evv, matrix vv, int n, int p)
{
    int i, j, k, ban, kmax=753;
    double rmax, rmin, rrq;
    vector y, x, u;
    matrix a;
    scalar ev, s;
    
    y = matinit(n,1);
    x = matinit(n,1);
    a = matinit(n,n);
    u = matinit(n,1);
    
    if (evv == NULL) evv = matinit(n,1);
    if (vv  == NULL) vv  = matinit(n,n);
    if (p <= 0) p = n;
    
    /* save original to a */
    matcpy(a0, n, n, a);
    
    for (ban=1; ban<=p; ban++) {           /* $B8GM-CM$,(B m $B8DF@$i$l$k$^$GH?I|(B */
        inival(x,n);                         /* $B=PH/CM$r@_Dj(B */
        for (k=0; k<kmax; k++) {
            matprd(a,n,n, x, n, 1, y);         /* Ax = y */
            rat(x,y,&rmin,&rmax,&rrq,n);       /* $B8GM-CM7W;;(B (1) */
            if ((rmax-rmin)<=eps)              /* $B<}B+H=Dj(B (1) */
                break;
            matcpy(y,n,1,x);                   /* y $B$r(B x $B$K0\$9(B */
            vectnormal(x,n);                   /* $BD9$5$r#1$K@55,2=(B */
        }
        matcpy(y,n,1,x);                     /* y $B$r(B x $B$K0\$9(B */
        vectnormal(x,n);                     /* $BD9$5$r#1$K@55,2=(B */
        s=0.0;                               /* $B8GM-CM$N@:EY2~NI(B */
        for (i=1; i<=n; i++)
            for (j=1; j<=n; j++)
                s += *(a0+offs(n,n,i,j)) * *(x+offs(n,1,i,1)) * *(x+offs(n,1,j,1));
        ev=s;
        
        *(evv+offs(n,1,ban,1)) = ev;         /* $B7k2L$rG[Ns$K=q$-9~$`(B */
        for (i=1; i<=n; i++)
            *(vv+offs(n,n,i,ban)) = *(x+offs(n,1,i,1));
        
        /* $B8:<!$9$k(B */
        for (i=1; i<=n; i++)
            for (j=1; j<=n; j++)
                *(a+offs(n,n,i,j)) = *(a+offs(n,n,i,j)) 
                    - ev * *(x+offs(n,1,i,1)) * *(x+offs(n,1,j,1));
    }

    free(y); free(x), free(u); free(a);
    return OK;
}

matrix eigenvector(matrix A, int n, int p)
{
    matrix ret = matinit(n,n);
    vector evv = matinit(n,1);
    
    eigen_pm(A,evv,ret,n,p);
    free (evv);
    return ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include "fileio_util.h"

int usage(char *prgname)
{
    printf("%s  - Principal component analysis\n", prgname);
    printf("Usage %s [options] [datafile]\n", prgname);
    printf("options: \n");
    printf(" -p      : number of eigenvalues to be extracted\n");
    printf(" -v      : calc eigenvalues by using coVariance matrix\n");
    printf(" -gw     : plots pca weights\n");
    printf(" -gs     : plots pca scores\n");
    printf(" -notext : inhibit text output\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int i, j, m=0, n=0, p=0, argn = 1;
    matinfo Ain;
    matrix A, At, eig, evec, mean, sd, Cov, pca, printEig, printEvec;
    double w, dn, dn1, dt, dt1;
    boolean corrflg = True, plotweightflg = False, plotscoreflg = False;
    boolean textflg = True;
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "-help")) {
            usage(argv[0]);
        } else if (keymatch(argv[argn], "-p")) {
            ++argn;
            if (argn == argc || sscanf(argv[argn], "%d", &p) != 1)
                usage(argv[0]);
        } else if (keymatch(argv[argn], "-v")) {
            corrflg = False;
        } else if (keymatch(argv[argn], "-gw")) {
            plotweightflg = True;
        } else if (keymatch(argv[argn], "-gs")) {
            plotscoreflg = True;
        } else if (keymatch(argv[argn], "-notext")) {
            textflg = False;
        } else {
            usage(argv[0]);
        }
        ++argn;
    }
    
    if (argn != argc) {
        Ain = matinfo_read(argv[argn]);
        ++argn;
    } else {
        Ain = matinfo_read("-");
    }
    if (argn != argc)
        usage(argv[0]);
    
    A = Ain->mat;
    n = Ain->n;
    m = Ain->m;
    
    if ((n == 0) || (m == 0)) usage(argv[0]);
    if (p == 0) p = m;
    
    eig = matinit(m,1);
    evec = matinit(m,m);
    mean = matinit(m,1);
    sd   = matinit(m,1);
    Cov  = matinit(m,m);
    
    for (j=1; j<=m; j++) {
        dt = 0.0; dt1 = 0.0;
        for (i=1; i<=n; i++) {
            w = *(A+offs(n,m,i,j));
            dn1 = (double)(i-1);
            dn  = (double)i;
            dt  = (dn1 * dt + w)/dn;
            dt1 = (dn1 * dt1 + w * w)/dn;
        }
        *(mean+offs(m,1,j,1)) = dt;
        *(sd+offs(m,1,j,1)) = dt1 - dt * dt;
    }
    
    for (j=1; j<=m; j++) {
        for (i=1; i<=n; i++) {
            *(A+offs(n,m,i,j)) -= *(mean+offs(m,1,j,1));
        }
    } 
    
    if (corrflg) {
        for (i=1; i<=n; i++) {
            for (j=1; j<=m; j++) {
                if ( *(sd+offs(m,1,j,1)) > eps ) {
                    *(A+offs(n,m,i,j)) /= sqrt(*(sd+offs(m,1,j,1)));
                } else {
                    *(A+offs(n,m,i,j)) = 0.0;
                }
            }
        }
    }
    At = matT(A,n,m);
    matprd(At,m,n,A,n,m,Cov);
    matARITH(Cov, m, m, (1.0/(double)n), 0.0);

    fprintf(stdout, argv[0], "principal component analysis");
    fprint_signature(stdout);
    fprintf(stdout, "### Rows: %d\n",n);
    fprintf(stdout, "### Columns: %d\n",m);
    fprintf(stdout, "### EigenvaluesToBeExtracted: %d\n",p);
    
    if (textflg) {
        /*
          fprintf(stdout, "n=%d, m=%d, p=%d\n",n,m,p);
        */
        fwritemat(stdout, mean, 1, m, "### means:\n", "%.6g ");
        fwritemat(stdout, sd,   1, m, "### variances:\n",  "%.6g ");
        
        fwritemat(stdout, Cov, m, m, corrflg ? "### Correlation:\n" :
                  "### Covariance:\n", "%.6g ");
    }
    
    free(At); free(sd); free(mean); 
    eigen_pm(Cov,eig,evec,m,p);
    
    if (textflg) {
        if ( p == m ){
            fwritemat(stdout, eig,  m,1, "### EigenValues:\n","%.6g ");
            fwritemat(stdout, evec, m,m, "### EigenVectors:\n", "%.6g ");
        } else {
            printEig = matTrim(eig, m,1,1,1,p,1);
            printEvec = matTrim(evec, m,m, 1,1, m,p);
            fwritemat(stdout, printEig,  p,1, "### EigenValues:\n","%.6g ");
            fwritemat(stdout, printEvec, m,p, "### EigenVectors:\n", "%.6g ");
        }
    }
    if (plotweightflg) {
        pca = matinit(m,p);
        for (i=1; i<=m; i++) 
            for (j=1; j<=p; j++) 
                *(pca+offs(m,p,i,j)) = *(evec+offs(m,m,i,j));
        plot2d(stdout, pca, m, p, "### PCA weight");
        free(pca);
    }
    
    if (plotscoreflg) {
        pca = matinit(n,m);
        matprd(A, n, m, evec, m, m, pca);
        plot2d(stdout, pca, n, m, "### Result of PCA");
        if (textflg) {
            fwritemat(stdout, pca, n,m, "\n### Principal Components:\n", "%.6g ");
        }
    }
    return OK;
}
#endif STANDALONE
