/*
 * @(#)gram_schmidt - matrix orthogonization
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

int gram_schmidt (matrix A, int n, int m, matrix Ret)
{
    if ( Ret == NULL ){
	fprintf (stderr, "Invalid matrix in gram_schmidt()\n");
	exit (EXIT_FAILURE);
    }
    if ( A == NULL ){
	fprintf (stderr, "Invalid matrix in gram_schmidt()\n");
	exit (EXIT_FAILURE);
    }
    Ret = Gram_Schmidt(A,n,m);
    return OK;
}

int GRAM_SCHMIDT (matrix A, int n, int m)
{
    matrix Tmp = matinit(n,m);

    if ( A == NULL ){
	fprintf (stderr, "Invalid matrix in GRAM_SCHMIDT()\n");
	exit (EXIT_FAILURE);
    }

    Tmp = Gram_Schmidt(A,n,m);
    matcpy(Tmp,n,m,A);
    free(Tmp);
    return OK;
}

matrix Gram_Schmidt (matrix A, int n, int m)
{
    int i,j, j2;
    double sum, len, prd;
    matrix Ret = matinit(n,m);

    if ( A == NULL ){
	fprintf (stderr, "Invalid matrix in Gram_Schmidt()\n");
	exit (EXIT_FAILURE);
    }

    for (j=1; j<=m; j++) {
	for (i=1; i<=n; i++) {
	    *(Ret+offs(n,m,i,j)) = *(A+offs(n,m,i,j));
	}
	for (j2=1; j2<j; j2++) {
	    prd = 0.0;
	    for (i=1; i<=n; i++) {
		prd += *(Ret+offs(n,m,i,j)) * *(Ret+offs(n,m,i,j2));
	    }
	    if ( prd == 0.0 ) {
		fprintf(stderr, "Data matrix may be an singluar matrix in Gram_Schmidt().\n");
		fprintf(stderr, "The inner product between %dth and %dth vector is zero\n", j, j2);
		exit (EXIT_FAILURE);
	    }
	    for (i=1; i<=n; i++) {
	    *(Ret+offs(n,m,i,j)) -= prd * *(Ret+offs(n,m,i,j2));
	    }
	}

	sum = 0.0;
	for (i=1; i<=n; i++) {
	    sum += *(Ret+offs(n,m,i,j)) * *(Ret+offs(n,m,i,j));
	}
	len = sqrt(sum);
	if (len < eps ) { len = 1.0; }
	for (i=1; i<=n; i++) {
	    *(Ret+offs(n,m,i,j)) /= len;
	}
    }
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *progname)
{
    fprintf(stderr,"%s : return Gram-Schmidts' orthogonalization\n", progname);
    fprintf(stderr,"Usage:\n%s [datafile]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matinfo In;
    matrix Out;
    char *filename;
    int argn=1;

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) { usage(argv[0]); } 
        else if (keymatch(argv[argn], "-h")) { usage(argv[0]); } 
        else { usage(argv[0]); }
        argn++;
    }

    if (argn != argc) { 
        if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory in main() of gram_schmidt\n"); 
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy in main() of gram_schmidt\n");
            exit (EXIT_FAILURE);
        }
        filename[strlen(argv[argn])] = '\0';
        argn++;
    } else {
        filename = (char *)malloc(2);
        filename[0]='-'; filename[1]='\0';
    }
    if (argn != argc) usage(argv[0]);
    In = matinfo_read(filename);
    if ( (In->n == 0) || (In->m == 0) ) usage(argv[0]);
    Out = Gram_Schmidt(In->mat, In->n, In->m);

    fprintf(stdout, "### ProgramName: %s %s (Gram Schmidt orthogonization)\n", argv[0], filename);
    fprint_signature(stdout);
    fwritemat(stdout, Out, In->n, In->m, "", "");
    return OK;
}
#endif STANDALONE
