/*
 * Author: Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 * $Id: matdiag.c,v 1.3 2001/06/24 03:29:31 asakawa Exp $
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

int matdiag (matrix A, int n, vector Ret)
{
    int i;

    if (Ret == NULL) {
	fprintf(stderr, "Invalid vector in matdiag()\n");
	exit (EXIT_FAILURE);
    }

    if (A == NULL) {
	fprintf(stderr, "Invalid matrix in matdiag()\n");
	exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
	*(Ret+offs(n,1,i,1)) = *(A+offs(n,n,i,i));
    }
    return OK;
}

vector matDiag (matrix A, int n)
{
    int i;
    vector ret = matinit(n,1);

    if (A == NULL) {
        fprintf(stderr, "Could not allocate memory in diag()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
        *(ret+offs(n,1,i,1)) = *(A+offs(n,n,i,i));
    }
    return ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <time.h>
#include "fileio_util.h"

int usage(char *progname)
{
    fprintf(stderr,"%s - return a diagonal vector for a matrix\n", progname);
    fprintf(stderr,"Usage:\n%s [file]\n", progname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    matinfo Ain;
    char *filename;
    matrix Out;
    int argn=1;

    while (argn < argc && argv[argn][0] == '-') {
        if (keymatch(argv[argn], "--help")) { usage(argv[0]); } 
        else if (keymatch(argv[argn], "-h")) { usage(argv[0]); } 
        else { usage(argv[0]); }
        argn++;
    }

    if (argn != argc) { 
        if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
            fprintf(stderr, "Could not allocate memory in main() of matnormalize\n"); 
            exit (EXIT_FAILURE);
        }
        if (strcpy(filename,argv[argn]) == NULL) {
            fprintf(stderr, "Could not strcpy in matnormalize()\n");
            exit (EXIT_FAILURE);
        }
        filename[strlen(argv[argn])] = '\0';
        argn++;
    } else {
        filename = (char *)malloc(2);
        filename[0]='-'; filename[1]='\0';
    }
    if (argn != argc) usage(argv[0]);
    Ain = matinfo_read(filename);
    if ( (Ain->n == 0) || (Ain->m == 0) ) usage(argv[0]);
    if ( Ain->n != Ain->m ) usage(argv[0]);

    fprintf(stdout, "### ProgramName: %s %s\n", argv[0], filename);
    fprint_signature(stdout);
    
    Out = matDiag(Ain->mat,Ain->n);
    fwritemat(stdout, Out, Ain->n, 1, "", "");
    return OK;
}
#endif STANDALONE
