/*
 * @(#)invmat.c - matrix operate function 1.02 21-07-1990
 * by Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 */
#include <stdlib.h>
#include "matrix_util.h"

void swap(scalar a, scalar b)
{
    scalar w;
    w = a; a = b; b = w;
}

int matinv(matrix A, int n, scalar *det, matrix ret)
{
    int i,j,k, immx;
    double dmax,di,piv,w;
    matrix work;
    
    *det = 1.0;
    work = matinit(n,n);
    matcpy(A,n,n,work);
    
    for (i=1;i<=n;i++)
        *(ret+offs(n,n,i,i)) = 1.0;
    
    for (i=1;i<=n;i++) {
        immx = i;
        if (immx != n) {
            dmax = fabs(*(work+offs(n,n,i,i)));
            for (j=i+1;j<=n;j++) {
                di = fabs(*(work+offs(n,n,j,i)));
                if (di > dmax) {
                    immx = j; dmax = di;
                }
            }
            if (immx != i) {
                for (j=1; j<=n; j++) {
                    swap(*(work+offs(n,n,i,j)), *(work+offs(n,n,immx,j)));
                    swap(*(ret+offs(n,n,i,j)), *(ret+offs(n,n,immx,j)));
                }
            }
        }
        piv = *(work+offs(n,n,i,i));
        if (fabs(piv) < eps) {
            fprintf(stderr,"Singular Marix!!\n");
            fprintf(stderr,"piv=%f\n",piv);
            fprintf(stderr,"fabs(piv)=%f\n",fabs(piv));
            fprintf(stderr,"eps=%f\n",eps);
            exit (EXIT_FAILURE);
        }
        *det *= piv;
        if (immx != i) *det *= -1.0;
        for (j=1; j<=n; j++) {
            *(work+offs(n,n,i,j)) /= piv;
            *(ret+offs(n,n,i,j)) /= piv;
        }
        for (j=1; j<=n; j++) {
            if (j != i) {
                w = *(work+offs(n,n,j,i));
                for (k=1;k<=n;k++) {
                    *(work+offs(n,n,j,k)) -= *(work+offs(n,n,i,k)) * w;
                    *(ret+offs(n,n,j,k)) -= *(ret+offs(n,n,i,k)) * w;
                }
            }
        }
    } 
    free (work);
    return OK;
}

/* return inverse matrix & inverse matrix     */
/* A data matrix ( input )                    */
/* n size of A ( input )                      */
/* *det determinant of A ( output )           */
matrix matInv(matrix A, int n, scalar *det)
{
    int i,j,k,immx;
    double dmax,di,piv,w;
    matrix ret, work;
    
    *det = 1.0;
    ret = matinit(n,n);
    work = matinit(n,n);
    matcpy(A,n,n,work);
    
    for (i=1;i<=n;i++)        
        *(ret+offs(n,n,i,i)) = 1.0;
    
    for (i=1;i<=n;i++) {
        immx = i;
        if (immx != n) {
            dmax = fabs(*(work+offs(n,n,i,i)));
            for (j=i+1;j<=n;j++) {
                di = fabs(*(work+offs(n,n,j,i)));
                if (di > dmax) {
                    immx = j; dmax = di;
                }
            }
            if (immx != i) {
                for (j=1;j<=n;j++) {
                    swap(*(work+offs(n,n,i,j)), *(work+offs(n,n,immx,j)));
                    swap(*(ret+offs(n,n,i,j)), *(ret+offs(n,n,immx,j)));
                }
            }
        }
        piv = *(work+offs(n,n,i,i));
        if (fabs(piv) < eps) {
            fprintf(stderr,"matInv: Singular Marix!!\n");
            fprintf(stderr,"piv=%f\n",piv);
            fprintf(stderr,"fabs(piv)=%f\n",fabs(piv));
            fprintf(stderr,"eps=%f\n",eps);
            exit (EXIT_FAILURE);
        }
        *det *= piv;
        if (immx != i) *det *= -1.0;
        for (j=1;j<=n;j++) {
            *(work+offs(n,n,i,j)) /= piv;
            *(ret+offs(n,n,i,j)) /= piv;
        }
        for (j=1;j<=n;j++) {
            if (j != i) {
                w = *(work+offs(n,n,j,i));
                for (k=1;k<=n;k++) {
                    *(work+offs(n,n,j,k)) -= *(work+offs(n,n,i,k)) * w;
                    *(ret+offs(n,n,j,k)) -= *(ret+offs(n,n,i,k)) * w;
                }
            }
        }
    }
    free (work);
    return ret;
}

scalar determinant(matrix A, int m)
{
    scalar det;
    matrix work;
    
    work = matInv(A,m,&det);
    free (work);
    return det;
}

#ifdef STANDALONE
#include <stdlib.h>
#include "fileio_util.h"

int usage(void)
{
    printf("matinv: return inverse matrix\n");
    printf("Usage:\nmatinv [datafile]\n");
    printf("Arguments: \n");
    printf(" datafile : data file name\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int n = 0, argn = 1;
    matinfo Ain;
    matrix A, X;
    scalar det;
    
    while (argn<argc && argv[argn][0] == '-' && argv[argn][1] != '\0'){
        if (keymatch(argv[argn], "--help")) { usage(); }
        else if (keymatch(argv[argn], "-h")) { usage(); }
        argn++;
    }
    
    if (argn != argc) Ain = matinfo_read(argv[argn]);
    else              Ain = matinfo_read("-");
    
    if (Ain->n != Ain->m) {
        fileio_util_perror("matinv(main): data is not square matrix");
        exit (EXIT_FAILURE);
    }
    n = Ain->n; A = Ain->mat;
    if (n == 0) usage();
    
    X = matInv(A, n, &det);
    
    fprintf(stdout, "### ProgramName: matinv %s\n", (argn!=argc) ? argv[argn]:"-");
    fprint_signature(stdout);
    fprintf(stdout, "### DataSize: %d\n",n);
    fprintf(stdout, "### determinant: %.7g\n", det);

    fwritemat(stdout, X, n, n, "", "%.7g ");
    return EXIT_SUCCESS;
}
#endif STANDALONE
