#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "general.h"
#include "matrix_util.h"
#include "fileio_util.h"

int matmask(matrix In, int tn, int tm, matrix Mask, int mn, int mm, matrix Ret)
{
    if (Ret == NULL) {
        fprintf(stderr,"Invalid target matrix in matmask()\n");
        exit (EXIT_FAILURE);
    }
    Ret = matMask(In, tn, tm, Mask, mn, mm);
    return OK;
}

int matMASK(matrix In, int tn, int tm, matrix Mask, int mn, int mm)  
{
    int i,j;
    
    if (In == NULL) {
        fprintf(stderr,"Invalid input matrix in matMask()\n");
        exit (EXIT_FAILURE);
    }
    if (Mask == NULL) {
        fprintf(stderr,"Invalid mask matrix in matMask()\n");
        exit (EXIT_FAILURE);
    }
    if (tn != mn) {
        fprintf(stderr,"Row size(%d,%d) differ between In and Mask in matMask()\n",tn,mn);
        exit (EXIT_FAILURE);
    }
    if (tm != mm) {
        fprintf(stderr,"Column size(%d,%d) differ between In and Mask in matMask()\n",tm,mm);
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=tn; i++) {
        for (j=1; j<=tm; j++){
            *(In+offs(tn,tm,i,j)) *= *(Mask+offs(mn,mm,i,j));
        }
    }
    return OK;
}

matrix matMask(matrix In, int tn, int tm, matrix Mask, int mn, int mm)  
{
    matrix Ret;
    int i,j;
    
    if (In == NULL) {
        fprintf(stderr,"Invalid input matrix in matMask()\n");
        exit (EXIT_FAILURE);
    }
    if (Mask == NULL) {
        fprintf(stderr,"Invalid mask matrix in matMask()\n");
        exit (EXIT_FAILURE);
    }
    if (tn != mn) {
        fprintf(stderr,"Row size(%d,%d) differ between In and Mask in matMask()\n",tn,mn);
        exit (EXIT_FAILURE);
    }
    if (tm != mm) {
        fprintf(stderr,"Column size(%d,%d) differ between In and Mask in matMask()\n",tm,mm);
        exit (EXIT_FAILURE);
    }
    if ((Ret = matinit(tn,tm)) == NULL) {
        fprintf(stderr,"Could not allocate memory in matMask()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=tn; i++) {
        for (j=1; j<=tm; j++){
            *(Ret+offs(tn,tm,i,j)) = *(In+offs(tn,tm,i,j)) * *(Mask+offs(mn,mm,i,j));
        }
    }
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>
#include <time.h>
long loadingtime;

int usage(char *prgname)
{
    fprintf(stderr,"%s\n", prgname);
    fprintf(stderr,"Usage:\n%s Input [Mask]\n", prgname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    matinfo Ain, Bin;
    matrix Out;

    if ((argc != 2) && (argc != 3)) {
        usage(argv[0]);
    } 

    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) usage(argv[0]);
        if (keymatch(argv[argn], "-h")) usage(argv[0]);
        ++argn;
    }
    
    Ain = matinfo_read(argv[1]);
    if (argc == 3) {
	Bin = matinfo_read(argv[2]);
    } else {
	Bin = matinfo_read("-");
    }
    Out = matMask(Ain->mat,Ain->n,Ain->m,Bin->mat,Bin->n,Bin->m);

    fprintf(stdout, "### ProgramName: %s %s %s\n", argv[0], 
	    argv[1], (argc==3)?argv[2] : "-");
    fprint_signature(stdout);

    fwritemat(stdout, Out, Ain->n, Bin->m, "", "");
    return OK;
}
#endif STANDALONE
