/*
 * @(#)matrix_util.c - matrix operate function 1.02 21-07-1990
 * by Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 * revised at Jan.-30-1997 for Solaris 2.5.1.
 */
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include "general.h"
#include "fileio_util.h"
#include "matrix_util.h"

/*--- Initialize a matrix which has n rows and m columns ---*/
matrix matinit(int n, int m) 
{
    unsigned int size;
    matrix ret;

    if ( (n==0) || (m==0) ) {
        fprintf(stderr, "Invalid matrix size in matinit()\n");
        exit (EXIT_FAILURE);
    }
    
    size = sizedef(n,m);
    if ((ret = (matrix)calloc(size, sizeof(scalar))) == NULL){
        fprintf(stderr, "Could not allocate memory in matinit()\n");
        exit (EXIT_FAILURE);
    }
    return ret;
}

/*--- Initialize an integer matrix which has n rows and m columns ---*/
int *intmatinit(int n, int m) 
{
    unsigned int size;
    int *ret;
    size = (n * m);
    
    if ((ret = (int *)calloc(size, sizeof(int))) == NULL){
        fileio_util_perror("ret=calloc() failed at intmatinit()");
        exit (EXIT_FAILURE);
    } 
    return ret;
}

/*--- Clear all elements of a matrix ---*/
int matclr(matrix w, int row, int col) 
{
    if ( (row == 0) || (col == 0) ) {
        fprintf(stderr, "Invalid matrix size at matclr()\n");
        exit (EXIT_FAILURE);
    }
    if ( memset(w, '\0', row *col * sizeof(scalar)) == NULL) {
        fileio_util_perror("memset() failed at matclr()");
    }
    return OK;
}

/* initialize a vector */
vector vectinit(int n) 
{
    vector ret;

    if ( n <= 0 ) {
        fprintf(stderr, "Invalid size(%d) in vectinit()\n", n);
        exit (EXIT_FAILURE);
    }
    if ((ret = matinit(n,1)) == NULL) {
        fileio_util_perror("ret=matinit() failed at vectinit()");
        exit (EXIT_FAILURE);
    }
    return ret;
}


/* return inner product between two vectors */
double vectinnerprd(vector a, vector b, int n)
{
    int i;
    double ret = 0.0;

    if ( n <= 0 ) {
        fprintf(stderr, "Invalid size(%d) at vectinnerprd()\n", n);
        exit (EXIT_FAILURE);
    }
    if ( a == NULL ) {
        fprintf(stderr, "Invalid vector a at vectinnerprd()\n");
        exit (EXIT_FAILURE);
    }
    if ( b == NULL ) {
        fprintf(stderr, "Invalid vector b at vectinnerprd()\n");
        exit (EXIT_FAILURE);
    }
    
    for (i=1; i<=n; i++)
        ret += *(a+offs(n,1,i,1)) * *(b+offs(n,1,i,1));
    return ret;
}

int vectouterprd(vector a, vector b, int n, matrix Out)
{
    int i,j;
    
    if ((a == NULL) || (b == NULL)) {
        fprintf(stderr, "Vector a or b is NULL at vectouterprd()\n");
        exit (EXIT_FAILURE);
    }
    if (Out == NULL) {
        fprintf(stderr, "Matrix Out is NULL at vectouterprd()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++)
        for (j=1; j<=n; j++)
            *(Out+offs(n,n,i,j)) += *(a+offs(n,1,i,1)) * *(b+offs(n,1,j,1));
    return OK;
}

matrix vectOuterprd(vector a, vector b, int n)
{
    matrix Ret;
    
    if ((a == NULL) || (b == NULL)) {
        fprintf(stderr, "Vector a or b is invalid at vectOuterprd()\n");
        exit (EXIT_FAILURE);
    }
    if ( (Ret = matinit(n,n)) == NULL ) {
        fileio_util_perror("Ret=matinit() failed at vectOuterprd()");
        exit (EXIT_FAILURE);
    }
    vectouterprd(a, b, n, Ret);
    return Ret;
}

/* normalization of a vector whose length is 1.0 */
int vectnormal(vector x, int n)
{
    int i;
    double s;
    
    s = sqrt(vectinnerprd(x,x,n));
    if ( s < eps ) {
        fprintf(stderr, "Value of inner product is too small at vectnormal()\n");
        exit (EXIT_FAILURE);
    }
    if ( isnan(s) != 0 ) {
        fprintf(stderr, "Invalid value of inner product at vectnormal()\n");
        exit (EXIT_FAILURE);
    }
    for (i=1; i<=n; i++) {
        *(x+offs(n,1,i,1)) /= s; 
    }
    return OK;
}
