/*
 *      @(#)matsq - matrix operate function 1.02 (Shinichi Asakawa) 21-07-1990
 */
#include <stdlib.h>
#include "matrix_util.h"
#include "fileio_util.h"

/* Ret = A^{t}A */
int matsq (matrix A, int n, int m, matrix Ret)
{
    int i,j,k;

    if ( A == NULL ) {
	fprintf (stderr, "Invalid matrix in matsq()\n");
	exit (EXIT_FAILURE);
    }

    if ( Ret == NULL ) {
	fprintf(stderr, "Invalid Ret matrix in matsq()\n");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=m; i++)
	for (j=1; j<=m; j++)
	    for (k=1; k<=n; k++)
		*(Ret+offs(m,m,i,j)) += *(A+offs(n,m,k,i)) * *(A+offs(n,m,k,j));
    return OK;
}

/* A = A^{t}A */
int matSQ (matrix A, int n, int m)
{
    int i,j,k;
    matrix Wrk = matinit(m,m);

    if ( n != m ) {
	fprintf (stderr, "Can not compute (row!=col) in matSQ()\n");
	exit (EXIT_FAILURE);
    }
    if ( A == NULL ) {
	fprintf (stderr, "Invalid matrix in matSQ()\n");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=m; i++)
	for (j=1; j<=m; j++)
	    for (k=1; k<=n; k++)
		*(Wrk+offs(m,m,i,j)) += *(A+offs(n,m,k,i)) * *(A+offs(n,m,k,j));
    matcpy(Wrk,n,n,A);
    free(Wrk);
    return OK;
}

/* return A^{t}A */
matrix matSq (matrix A, int n, int m)
{
    int i,j,k;
    matrix ret;

    if ( A == NULL ) {
	fprintf(stderr, "Invalid matrix in matSq()\n");
	exit (EXIT_FAILURE);
    }

    if ((ret = matinit(m,m)) == NULL) {
	fprintf(stderr, "Could not allocate memory in matSq()\n");
	exit (EXIT_FAILURE);
    }

    for (i=1; i<=m; i++)
	for (j=1; j<=m; j++)
	    for (k=1; k<=n; k++)
		*(ret+offs(m,m,i,j)) += *(A+offs(n,m,k,i)) * *(A+offs(n,m,k,j));
    return ret;
}

int matsqinv(matrix A, int n, int m, matrix Ret)
{
    matrix AtA;
    scalar det;

    if ( A == NULL ) {
	fprintf(stderr, "Invalid original matrix in matsqinv()\n");
	exit (EXIT_FAILURE);
    }

    if (Ret == NULL) {
	fprintf(stderr, "Invalid Ret matrix in matsqinv()\n");
	exit (EXIT_FAILURE);
    }
    AtA = matSq(A,n,m);
    Ret = matInv(AtA,m, &det);
    free(AtA);
    return OK;
}

int matSQINV(matrix A, int n, int m)
{
    matrix AtA, Wrk;
    scalar det;

    if ( n != m ) {
	fprintf (stderr, "Can not compute (row!=col) in matSQINV()\n");
	exit (EXIT_FAILURE);
    }
	
    if ( A == NULL ) {
	fprintf(stderr, "Invalid matrix in matSQINV()\n");
	exit (EXIT_FAILURE);
    }

    AtA = matSq(A,n,m);
    Wrk = matInv(AtA,m, &det);
    matcpy(Wrk,n,n,A);
    free(AtA);    free(Wrk);
    return OK;
}


matrix matSqinv(matrix A, int n, int m)
{
    matrix ret, AtA;
    scalar det;

    AtA = matSq(A,n,m);
    ret = matInv(AtA,m, &det);
    free(AtA);
    return ret;
}

/* Ret = {A^{t}A}^{-1}A^{-t} */
int matsqinvmatt(matrix A, int n, int m, matrix Ret)
{
    matrix wk1, wk2;

    if ( A == NULL ) {
	fprintf(stderr, "Invalid matrix in matSqinvmat()\n");
	exit (EXIT_FAILURE);
    }
    if ( Ret == NULL ) {
	fprintf(stderr, "Invalid Ret matrix in matSqinvmat()\n");
	exit (EXIT_FAILURE);
    }
    wk1 = matSqinv(A,n,m);
    wk2 = matT(A,n,m);
    matprd(wk1,m,m,wk2,m,n,Ret);
    free (wk1); free (wk2);
    return OK;
}

/* A = {A^{t}A}^{-1}A^{-t} */
int matSQINVMATT(matrix A, int n, int m)
{
    matrix wk1, wk2, wk3;

    if ( A == NULL ) {
	fprintf(stderr, "Invalid matrix in matSQINVMAT()\n");
	exit (EXIT_FAILURE);
    }
    wk1 = matSqinv(A,n,m);
    wk2 = matT(A,n,m);
    wk3 = matPrd(wk1,m,m,wk2,m,n);
    matcpy (wk3,m,n, A);
    free (wk1); free (wk2); free (wk3);
    return OK;
}

/* return {A^{t}A}^{-1}A^{-t} */
matrix matSqinvmatt(matrix A, int n, int m)
{
    matrix Ret, wk1, wk2;

    if ( A == NULL ) {
	fprintf(stderr, "Invalid matrix in matSqinvmatt()\n");
	exit (EXIT_FAILURE);
    }    
    wk1 = matSqinv(A,n,m);
    wk2 = matT(A,n,m);
    Ret = matPrd(wk1,m,m,wk2,m,n);
    free (wk1); free (wk2);
    return Ret;
}

#ifdef STANDALONE
#include <stdlib.h>

int usage(char *prgname)
{
    fprintf(stderr,"%s return squared matrix\n", prgname);
    fprintf(stderr,"Usage:\n%s A\n", prgname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int argn=1;
    char *filename;
    matinfo In;
    matrix Out;
    FLAG IsFile = False;

    while (argn < argc) {
        if (keymatch(argv[argn], "--help")) { usage(argv[0]); } 
        else if (keymatch(argv[argn], "-h")) { usage(argv[0]); } 
        else {
	    if ((filename = (char *)malloc(strlen(argv[argn])+1)) == NULL) {
		fprintf(stderr, "Could not allocate memory in matsq()\n");
		exit (EXIT_FAILURE);
	    }
	    if (strcpy(filename,argv[argn]) == NULL) {
		fprintf(stderr, "Could not strcpy in matsq()\n");
		exit (EXIT_FAILURE);
	    }
	    filename[strlen(argv[argn])] = '\0';
	    In = matinfo_read(filename);
	    Out = matSq(In->mat, In->n, In->m);

	    fprintf(stdout, "### ProgramName: %s %s\n", argv[0], filename);
	    fprint_signature(stdout);
	    IsFile = fwritemat(stdout, Out, In->m, In->m, "", "");
	}
	argn++;
    }

    if (! IsFile) {
	In = matinfo_read("-");
	Out = matSq(In->mat, In->n, In->m);

	fprintf(stdout, "### ProgramName: %s %s\n", argv[0], "-");
	fprint_signature(stdout);
	return fwritemat(stdout, Out, In->m, In->m, "", "");
    }
    return OK;
}
#endif STANDALONE
