/*
 * Plot 2d diagram -- by Shinichi ASAKAWA <asakawa@twcu.ac.jp>
 */
#include <float.h>  /* for DBL_MAX       1.79769313486231570e+308 */
#include "matrix_util.h"
#include "fileio_util.h"

int plot2d(FILE *fp, matrix x, int n, int m, char *header)
{
#define PLANEY 31
#define PLANEX 51
    int i,j,k, nx,ny,ix,iy;
    int xaxis, yaxis;
    int plotx, ploty;
    double xval,yval,xmin, xmax;
    char plane[PLANEY][PLANEX];
    nx =PLANEX -1;
    ny =PLANEY -1;
    
    xmin = DBL_MAX;
    xmax = - xmin;
    
    for (i=1; i<=n; i++) {
        for (j=1; j<=m; j++) {
            xmin = min(xmin, *(x+offs(n,m,i,j)));
            xmax = max(xmax, *(x+offs(n,m,i,j)));
        }
    }
    if (xmax == xmin) return NG;

    if (xmin>=0.0) { 
	xmin = -xmax; 
    } else {
        if (-xmax< xmin) 
	    xmin = -xmax;
        else 
	    xmax = -xmin;
    }
    
    for (xaxis=1; xaxis<m; xaxis++) {
        for (yaxis=xaxis+1; yaxis<=m; yaxis++) {
            plotx = xaxis;
            ploty = yaxis;
            /*  
                for (j=2; j<=m; j++) {
                plotx =1;
                ploty =j;
            */
            for (i=1; i<=ny; i++) { /* fill space code to all the plane */
                for (k=1; k<=nx; k++) {
                    plane[i][k] = ' ';
		}
	    }
            for (i=1; i<=ny; i++) { /* draw a vertical axis */
                plane[i][1]='|';
                plane[i][nx] ='|';
            }
            for (k=1; k<=nx; k++) { /* draw a horizonal axis */
                plane[1][k] = '-';
                plane[ny][k] = '-';
            }
            plane[1][1]='+';        /* draw the original points */
            plane[1][nx]='+';
            plane[ny][1]='+';
            plane[ny][nx]='+';

            xval = (0.0 - xmin) /(xmax - xmin);
            yval = 1.0 - (0.0 - xmin) /(xmax -xmin);

            ix = xval * nx;
            if(ix >=1 && ix <=nx)
                for (i=1; i<=ny; i++)
                    plane[i][ix] = '+';
            iy = yval *ny;
            if(iy >=1 && iy <=ny)
                for (i=1; i<=nx; i++)
                    plane[iy][i] = '+';

            /*  data is from xmin to max, convert these data is from 0 to 1 */
            for (i=1; i<=n; i++) {
                xval =(*(x+offs(n,m,i,plotx)) - xmin)/(xmax - xmin);
                yval =1.0 - (*(x+offs(n,m,i,ploty)) - xmin)/(xmax - xmin);

                ix = xval *nx;
                if (ix <1)   ix = 1;
                if (ix >nx)  ix = nx;

                iy = yval * ny;
                if (iy <1)   iy = 1;
                if (iy >ny)  iy = ny;

                if ( i <= 9 ) plane[iy][ix] = i + '0';
                else if ((i>=10) && (i<36)) plane[iy][ix] = i + 'A' - 10;
                else if ((i>=36) && (i<=61)) plane[iy][ix] = i + 'a' - 36;
                else plane[iy][ix]='#';
            }
            fputs("\n",fp);

            for (i=1; i<=ny; i++) {
                for (k=1; k<=nx; k++) {
                    fputc(plane[i][k],fp);
		}
                fputs("\n",fp);
            }
            fputs("\n",fp);
            fprintf(fp,"Plot2d %s :  %d(x) x   %d(y)\n", header, plotx, ploty);
        }
    }
    return(1);
}

#ifdef STANDALONE
#include <stdlib.h>
#include <stdio.h>

int usage(char *prgname)
{
    printf("Usage %s [datafile] - plot 2d diagram\n", prgname);
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int n = 0, m = 0, argn = 1;
    matrix A;
    matinfo Ainfo;
    
    while (argn < argc && argv[argn][0] == '-' && argv[argn][1] != '\0') {
        if (keymatch(argv[argn], "--help")) {
            usage(argv[0]);
        }
        if (keymatch(argv[argn], "-h")) {
            usage(argv[0]);
        }
        ++argn;
    }
    if (argn != argc) {
        Ainfo = matinfo_read( argv[argn] );
    } else {
        Ainfo = matinfo_read("-");
    }
    A = Ainfo->mat;
    n = Ainfo->n; m = Ainfo->m;

    fprintf(stdout, "### ProgramName: %s - plot 2d diagram", argv[0]);
    fprint_signature(stdout);
    plot2d(stdout, A, n, m, "");
    return OK;
}
#endif STANDALONE
