/* Regression analysis - by Shinichi ASAKAWA <asakawa@twcu.ac.jp>

Copyright (c) 1999 Shinichi ASAKAWA <asakawa@twcu.ac.jp>

Disclaimer:  No guarantees of performance accompany this software,
nor is any responsibility assumed on the author.  */

#include <stdlib.h>
#include <math.h>
#include <string.h>
#include "matrix_util.h"
#include "fileio_util.h"

FLAG verbose=False;

int usage(void)
{
    fprintf(stderr,"reg - simple regression, solve y = Ax, return x vector\n");
    fprintf(stderr,"Usage reg [options] -A datafile -y datafile\n");
    fprintf(stderr,"  Arguments:\n");
    fprintf(stderr,"   -A datafile: filename of coefficient matrix, \n");
    fprintf(stderr,"   -y datafile: filename of target matrix,\n");
    fprintf(stderr,"  Options:\n");
    fprintf(stderr,"   -1 : add constant\n");
    fprintf(stderr,"   -v : verbose output\n");
    fprintf(stderr,"   -h : show this help\n");
    exit (EXIT_FAILURE);
}

int main(int argc, char **argv)
{
    int an=0, am=0, yn=0, ym=0, xn=0, xm=0, argn=1;
    matinfo Yin, Ain;
    matrix X, Y, A, Pred, Res, A1;
    char *fileA, *fileY;
    FLAG ADDCONST = False;

    while (argn<argc) {
        if (keymatch(argv[argn], "-help")) { usage(); }
        if (keymatch(argv[argn], "-1")) {
            ADDCONST = True;
        }
        if (keymatch(argv[argn], "-A")) {
            if ((fileA=(char *)malloc(strlen(argv[argn+1])+1)) == NULL){
                fileio_util_perror("malloc() ERROR at main() in reg"); 
                exit (EXIT_FAILURE);
            }
            strcpy(fileA, argv[argn+1]);
            fileA[strlen(argv[argn+1])]='\0';
            Ain = matinfo_read(fileA);
            an = Ain->n; am = Ain->m; A = Ain->mat;
        }
        if (keymatch(argv[argn], "-y")) {
            if ((fileY=(char *)malloc(strlen(argv[argn+1])+1)) == NULL){
                fileio_util_perror("malloc() ERROR at main() in reg"); 
                exit (EXIT_FAILURE);
            }
            if (strcpy(fileY, argv[argn+1]) == NULL) {
                fileio_util_perror("strcpy() ERROR at main() in reg"); 
                exit (EXIT_FAILURE);
            }
            fileY[strlen(argv[argn+1])]='\0';
            Yin = matinfo_read(fileY);
            yn = Yin->n; ym = Yin->m; Y = Yin->mat;
        }
        if (keymatch(argv[argn], "-v")) { verbose=True; }
        ++argn;
    }

    if ((an ==0) || (am==0) || (yn==0) || (ym==0)) usage();
    if (an != yn) usage();

    if (ADDCONST) {
        int i,j;
        A1 = matinit(an,am+1);
        for (i=1; i<=an; i++) {
            for (j=1; j<=am; j++) 
                *(A1+offs(an,am+1,i,j)) = *(A+offs(an,am,i,j));
            *(A1+offs(an,am+1,i,am+1)) = 1.0;
        }
        am++;
        free (A);
        A = A1;
    }
    
    xn = am; xm = ym;
    
    X = matPrd(matSqinvmatt(A,an,am),am,an,Y,yn,ym);

    fprintf(stdout, "### ProgramName: %s -A %s -y %s %s %s\n", argv[0], fileA, fileY, (ADDCONST) ? "-1" : "", verbose ? "-v" : "");
    fprint_signature(stdout);
    fprintf(stdout, "### Rows: %d\n",xn);
    fprintf(stdout, "### Columns: %d\n",xm);
    if (verbose) {
        fwritemat(stdout, X, xn, xm, "### x:\n", "");
        Pred = matPrd(A,an,am,X,xn,xm);
        fwritemat(stdout, Pred, yn, ym, "### Predictor:\n", "");
        Res = matSub(Y, yn,ym, Pred, yn, ym);
        fwritemat(stdout, Res, yn, ym, "### Residual:\n", "");
    } else {
        fwritemat(stdout, X, xn, xm, "", "%.6g ");
    }
    return OK;
}
