/***********************************************************
        binormal.c -- 2ʬ
***********************************************************/
#include <math.h>
#include "general.h"
#include "rand_gen.h"

/* random number according with 2 dimenstional normal distribution */
/*
  When W and X are in accordance with N(0,1^2) indenpendently, 
  then Y = a W + b X, Z = a W - b X are in accordance wiht
  binormal distribution, which variance is a^2 + b^2, and 
  it's correlation is a^2 - b^2
  */
void binormal_rand (double r, double *x, double *y) {
    double r1, r2, s;

    do {
        r1 = 2.0 * MTgenrand_f() - 1.0;
        r2 = 2.0 * MTgenrand_f() - 1.0;
        s = r1 * r1 + r2 * r2;
    } while (s > 1 || s == 0.0);
    s = - log(s) / s;
    r1 = sqrt((1.0 + r) * s) * r1;
    r2 = sqrt((1.0 - r) * s) * r2;
    *x = r1 + r2;  
    *y = r1 - r2;
}

/**********************************************************/
#ifdef STANDALONE
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

long loadingtime;
unsigned int SEED;

void Usage (char *prg) {
  fprintf(stderr,"Usage:\n");
  fprintf(stderr,"%s -n # -r # [-ux #][-uy #][-sx #][-sy #][-SEED #]\n",prg);
  fprintf(stderr,"options  -n: num of data to be generated(int)\n");
  fprintf(stderr,"         -r: correlation coefficient(float)\n");
  fprintf(stderr,"        -ux: mean X (float), default=0.0\n");
  fprintf(stderr,"        -uy: mean Y (float), default=0.0\n");
  fprintf(stderr,"        -sx: standard deviation of X(float), default=1.0\n");
  fprintf(stderr,"        -sy: standard deviation of Y(float), default=1.0\n");
  fprintf(stderr,"        -SEED: seed of random number(long int)\n");
  exit (EXIT_FAILURE);
}

int main (int argc, char **argv) {
    int i,t, max = 0;
    double x, y;
    double corr = 0.0;
    double ux = 0.0, uy = 0.0;
    double sx = 1.0, sy = 1.0;

    SEED = (unsigned)time(&loadingtime);

    for (t=1; t<argc; t++){
        if (argv[t][0] == '-'){
            if (strcmp(argv[t],"-n") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                max = (unsigned)atoi(argv[t]);
            } else if (strcmp(argv[t],"-r") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                corr = atof(argv[t]); 
            } else if (strcmp(argv[t],"-ux") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                ux = atof(argv[t]); 
            } else if (strcmp(argv[t],"-uy") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                uy = atof(argv[t]); 
            } else if (strcmp(argv[t],"-sx") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                sx = atof(argv[t]); 
            } else if (strcmp(argv[t],"-sy") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                sy = atof(argv[t]); 
            } else if (strcmp(argv[t],"-SEED") == 0) {
                if (argv[++t] == NULL)  Usage(argv[0]);
                SEED = atof(argv[t]); 
            } else if (strcmp(argv[t],"-h") == 0) {
                Usage(argv[0]);
                exit (EXIT_FAILURE);
            }
        }
    }
    if (sx == 0.0) {
        fprintf(stderr, "Illegal args(sx)\n");
        Usage(argv[0]);
    }
    if (sy == 0.0) {
        fprintf(stderr, "Illegal args(sy)\n");
        Usage(argv[0]);
    }
    if (max == 0)     Usage(argv[0]);

    init_random(SEED);

    fprintf(stdout,"### ProgramName: %s -n %d -r %g -ux %g -uy %g -sx %g -sy %g -SEED %d\n", 
                       argv[0], max, corr, ux, uy, sx, sy, get_seed());
    fprintf(stdout, "### Creator: Shinichi ASAKAWA <asakawa@twcu.ac.jp>\n");
    (void)time(&loadingtime);
    fprintf(stdout, "### CreationDate: %s", ctime(&loadingtime));

    for (i = 0; i < max; i++) {
        /*        rand_gauss2d(corr, &x, &y); */
        binormal_rand(corr, &x, &y);
        printf("%10.5f %10.5f\n", x * sx +ux, y * sy +uy);
    }
    return EXIT_SUCCESS;
}
#endif STANDALONE
